# Copyright 2015 Midokura SARL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron.db.migration.models import head

import midonet.neutron.db.bgp_speaker_router_insertion_model  # noqa
import midonet.neutron.db.data_state_db  # noqa
import midonet.neutron.db.data_version_db  # noqa
import midonet.neutron.db.gateway_device  # noqa
import midonet.neutron.db.logging_resource_model  # noqa
import midonet.neutron.db.port_binding_db  # noqa
import midonet.neutron.db.provider_network_db  # noqa
import midonet.neutron.db.task_db  # noqa


def get_metadata():
    return head.model_base.BASEV2.metadata
